@@ -10,7 +10,7 @@ use Class::AutoDB::Connect;
 use Class::AutoDB::Database;
 use Class::AutoDB::Registry;
 use Class::AutoDB::RegistryDiff;
-our $VERSION = '1.29_07';
+our $VERSION = '1.10';
 $VERSION=eval $VERSION;		# I think this is the accepted idiom..
 
 # NG 09-11-24: move Database first so AutoClass::get will not mask Database::get
@@ -30,14 +30,12 @@ Class::AutoClass::declare;
 
 use vars qw($AUTO_REGISTRY);	# TODO: move to Globals
 $AUTO_REGISTRY=new Class::AutoDB::Registry;
-our $GLOBALS=Class::AutoDB::Globals->instance();
 
-# usual case is compile time registration: autodb not yet set
 sub auto_register {
   my($args)=@_;
-  my $autodb=$GLOBALS->autodb;
-  $autodb? $autodb->register($args): $AUTO_REGISTRY->register($args);
+  $AUTO_REGISTRY->register($args);
 }
+our $GLOBALS=Class::AutoDB::Globals->instance();
 
 sub _init_self {
   my($self,$class,$args)=@_;
@@ -45,12 +43,7 @@ sub _init_self {
   $GLOBALS->autodb($self) unless $GLOBALS->autodb;
   return unless $self->is_connected; # connection handled in Class::AutoDB::Connect
   # NG 09-12-05: alter, index needed by register. find, get no longer supported
-  # NG 11-01-07: register allowed to alter if -create set
-  # my($alter,$index)=@$args{qw(alter index)};
-  # $self->set(create_param=>$create,alter_param=>$alter,index_param=>$index);
-  my($create,$alter,$index)=@$args{qw(create alter index)};
-  # NG 11-01-07: setting alter_param harder than it looks 'cuz of default semantics.. 
-  $alter=($alter||$create)? 1: (!defined $alter? undef: 0);
+  my($alter,$index)=@$args{qw(alter index)};
   $self->set(alter_param=>$alter,index_param=>$index);
   $self->manage_registry($args);
   # NG 09-12-05: find, get params no longer supported
@@ -81,18 +74,6 @@ sub register {
   $self->manage_registry
     (new Hash::AutoHash::Args alter=>$self->alter_param,index=>$self->index_param);
 }
-# NG 10-09-16: realized some time ago that is_extant, is_deleted redundant since overloaded
-#              'bool' does same thing more easily, but forgot to comment them out from here
-# NG 10-09-07: added is_extant, is_deleted to support deleted objects
-# sub is_extant {
-#   my($self,$obj)=@_;
-#   Class::AutoDB::Serialize::is_extant($obj);
-# }
-# sub is_deleted {
-#   my($self,$obj)=@_;
-#   !Class::AutoDB::Serialize::is_extant($obj);
-# }
-
 # NG 09-11-24: extend put_object to put list of objects
 # NG 09-12-19: rewrote to avoid $obj->put for cleanup of user-object namespace
 sub put_objects {
@@ -130,7 +111,6 @@ sub put {
     # next if $obj==$registry;	# registry takes care of itself
     # NG 09-12-19: this line now needed because we stopped doing $obj->put
     #              (Oid has 'put' method that skips the store)
-    # NG 10-09-06: note that OidDeleted isa Oid
     next if UNIVERSAL::isa($obj,'Class::AutoDB::Oid');
     # NOTE: overload man page suggests comparing refaddrs instead of object.
     #       but comparing objects seems to work, provided $obj NOT Oid!!
@@ -146,21 +126,6 @@ sub put {
     $self->do_sql(@sql);
   }
 }
-# NG 10-09-06: added 'del' method
-sub del {
-  my $self=shift;
-  my $registry=$self->registry;
-  for my $obj (@_) {
-    next if UNIVERSAL::isa($obj,'Class::AutoDB::OidDeleted'); # already deleted
-    my $class=UNIVERSAL::isa($obj,'Class::AutoDB::Oid')? $obj->{_CLASS}: ref $obj;
-    my $oid=Class::AutoDB::Serialize::obj2oid($obj);
-    my $collections=$registry->class2collections($class);
-    $self->throw("trying to delete something non-persistent: $obj") unless $oid;
-    Class::AutoDB::Serialize::del($oid);      # delete from _AutoDB table
-    my @sql=map {$_->del($obj)} @$collections;         # generate SQL to store in collections
-    $self->do_sql(@sql);
-  }
-}
 # NG 09-11-29: added 'oid' method. first step towards cleaning namespace of user-objects 
 #              it's okay and useful to leave 'oid' method in Object and Oid for users' 
 #              convenience, but we should never call it
@@ -240,7 +205,7 @@ Class::AutoDB - Almost automatic object persistence coexisting with human-engine
 
 =head1 VERSION
 
-Version 1.29
+Version 1.10
 
 =head1 SYNOPSIS
 
@@ -256,9 +221,7 @@ Version 1.29
   Class::AutoClass::declare;
 
   ########################################
-  # code that uses persistent class
-
-  # create and store new objects
+  # code that uses persistent class - create and store new objects
   #
   use Class::AutoDB;
   use Person;
@@ -277,7 +240,8 @@ Version 1.29
   # store objects in database
   $autodb->put_objects;
 
-  # retrieve existing objects
+  ########################################
+  # code that uses persistent class - retrieve existing objects
   #
   use Class::AutoDB;
   use Person;
@@ -312,26 +276,6 @@ Version 1.29
         WHERE Dept.id=EmpDept.dept_id AND EmpDept.emp_id=Person.id 
         AND Person.name='Joe'));
 
-  ########################################
-  # new features in verion 1.20
-
-  # retrieve objects using SQL
-  # assuming the above database (with human-engineered tables Dept and EmpDept),
-  # this query retrieves Person objects for employees in the toy department
-  my @toy_persons=
-  $autodb->get
-    (sql=>qq(SELECT oid FROM Dept, EmpDept, Person 
-             WHERE Dept.id=EmpDept.dept_id AND EmpDept.emp_id=Person.id 
-             AND Dept.name='toy'));
-
-  # retrieve all objects
-  my @all_objects=$autodb->get;
-
-  # delete objects
-  #
-  $autodb->del(@males);                                  # delete the boys  
-
-
 =head1 DESCRIPTION
 
 This class works closely with L<Class::AutoClass> to provide almost
@@ -434,13 +378,6 @@ Person to have a separate Name object (which seems likely), it will
 work fine for Name to be non-persistent.  However, if you want Persons
 to share Names, you should declare Name to be persistent.
 
-As of version 1.20, it is possible to delete persistent objects using
-the 'del' method. Deleting an object removes it from memory and the
-database. However, it does not remove references to the deleted
-objects that may be held in program variable or other objects.  In
-boolean context, a deleted object evaluates to undef; in a string
-context it evaluates to an empty string. 
-
 =head2 Defining a persistent class
 
 You declare a class to be persistent by defining the %AUTODB variable
@@ -813,8 +750,8 @@ object.
 When an object is retrieved from the database, the system does NOT
 immediately process the oids it contains. Instead, the system waits
 until the program tries to access the referenced object at which point
-it automatically retrieves the object from the database. In a future
-release, options may be provided to retrieve oids earlier.
+it automatically retrieves the object from the database. Options are
+provided to retrieve oids earlier.
 
 If a program retrieves the same oid multiple times, the system short
 circuits the database access and ensures that only one copy of the
@@ -827,28 +764,6 @@ software consults this hash when asked to retrieve an object; if the
 object is already in memory, it is returned without even going to the
 database.
 
-Deleting an object converts its in-memory representation to a special
-kind of deleted-oid, removes the object from all its collections, and
-changes its BLOB in the object column of the _AutoDB table to NULL. We
-leave the object's oid in the _AutoDB table to allow better error
-handling on subsequent attempts to access the object.  See L<"Object
-deletion details"> for further discussion.
-
-=head3 UNIVERSAL methods
-
-UNIVERSAL, the base class of everything, defines four methods: isa,
-DOES, can, and VERSION.  Logically these are class methods, because
-they return the same answer for all objects of a given class.  In
-practice, programmers frequently invoke these on objects.
-
-When invoked on an oid, the system intercepts the call and
-redispatches the method to the oid's real class.  The system does not
-retrieve the object from the database.
-
-When invoked on a deleted-oid, the system intercepts the call and
-generates an error to be consistent with how all other method
-invocations are handled.  See L<"Object deletion details">.
-
 =head3 Data::Dumper details
 
 We rely on "freezer" and "toaster" methods to convert references to
@@ -867,8 +782,6 @@ copy of the object.
 classes do and emits a call to the method even if the object in
 question can't do it.
 
-=back
-
 We modified Data::Dumper to fix these problems. To avoid any confusion
 with the official version of the code, we renamed the modified version
 Class::AutoDB::Dumper and include it in our code tree.
@@ -883,73 +796,7 @@ ignored. The Data::Dumper docs claim that the code will fall back to
 the pure Perl implementation if the C version is not available.  We
 haven't checked this claim.
 
-=head3 Object deletion details
-
-Object deletion is a bit messy, since it is not a Perl concept.  In
-standard Perl, an object exists until all references to the object
-cease to exist.  As a result, the programmer never has to worry about
-stale references.  The situation changes when we introduce object
-deletion: it is now possible for a variable to contain a reference
-pointing to an object that has been deleted; likewise, an object can
-contain a reference (technically an Oid) pointing to a deleted object.
-
-The closest Perl analog is L<weak
-references|Scalar::Util/"weaken_REF">.  The standard Perl rule stated
-above -- "an object exists until all references to the object cease to
-exist" -- does not apply to weak references.  A weak reference can go
-stale (in other words, point to an object that no longer exists). When
-this happens, Perl sets the reference to undef, and any attempt to
-access the object via the stale reference generates an error.
-
-We adopt similar semantics for object deletion.  We do not convert
-stale references to undef (would require delving into Perl internals
--- doable but not worth the effort), but we detect attempts to invoke
-methods on deleted objects and 'confess' with appropriate error
-messages.  This complicates the application, but there seems to be no
-better choice.
-
-An application that uses object deletion extensively has to guard
-against attempts to invoke methods on deleted objects.  One way to
-do this is to test the object before accessing it.  For example, if
-$object contains a reference to an object that might be deleted,
-you might use this code snippet:
-
- if ($object) {   # $object will test 'true' if it still exists
-   # okay to invoke methods on $object here
- } else {           # 'else' $object has been deleted
-   # deal with deleted object here
- }
-
-To avoid such complications, we recommend using object deletion only
-in situations where you can confidently remove all references to the
-deleted objects.  For example, we use it when re-initializing entire
-data structures.
-
-The 'del' method operates on the in-memory and database
-representations of the objects being deleted. 
-
-It converts the in-memory representation to an OidDeleted, which is a
-special kind of Oid.  Any attempt to invoke application methods on an
-OidDeleted confesses.
-
-On the database, 'del' updates the object table (_AutoDB) and all
-tables implementing collections that contain the object.  In _AutoDB,
-'del' sets the 'object' column to NULL thereby removing the object's
-BLOB representation from the database; it leaves the object's oid in
-_AutoDB so we can detect future attempts to access the deleted object
-and generate an appropriate error message.  (If we were to delete the
-oid, it would look like the object was never stored, and future
-accesses would generate a misleading error message).  In collection
-tables, 'del' removes all rows representing the object; the code
-basically deletes the object from any table that 'put' would have put
-it in.
-
-'del' does B<not> remove references to the deleted object as this
-would require scanning the entire database.
-
-Queries ('get', 'find') will never retrieve a deleted object (although
-they may retrieve objects containing references to deleted object),
-and 'count' will not include deleted objects in its count.
+=back
 
 =head1 METHODS AND FUNCTIONS
 
@@ -1172,7 +1019,7 @@ automatically if the connection breaks.
 =head2 Queries
 
 The methods described here operate on Class::AutoDB objects.  See
-L<METHODS AND FUNCTIONS - Cursors|"Cursors"> for related methods operating on
+L<"METHODS AND FUNCTIONS - Cursors"> for related methods operating on
 Class::AutoDB::Cursor objects.
 
 =head3 get
@@ -1187,16 +1034,6 @@ Class::AutoDB::Cursor objects.
            -- OR --
            my $males=$autodb->get(collection=>'Person',
                                   query=>{name=>'Joe',sex=>'M'})
-           -- OR --
-           my @all_objects=$autodb->get
-           -- OR --
-           my $all_objects=$autodb->get
-           -- OR --
-           my @joe_bill=$autodb->get(sql=>qq(SELECT oid FROM Person 
-                                             WHERE name='Joe' OR name='Bill'))
-           -- OR --
-           my $joe_bill=$autodb->get(sql=>qq(SELECT oid FROM Person 
-                                             WHERE name='Joe' OR name='Bill'))
  Function: Execute query and return results
  Returns : list or ARRAY of objects satisfying query
  Args    : collection Name of collection being queried
@@ -1206,21 +1043,12 @@ Class::AutoDB::Cursor objects.
                       'list(object)', value must be simple scalar (string or 
                       integer). For 'object' or 'list(object)', value must be
                       persistent object
-           sql        raw SQL that selects oids of objects to be retrieved 
            other args interpreted as search_key=>value pairs
- Notes   : With no arguments, retrieves all objects.
-           Okay to include both search keys and SQL
 
 search_key=>value pairs are ANDed. For list types, the query is true if
 any element on list has the value. If a collection has a search key named
 'collection', you must use an explicit 'query' arg to include it in a query.
 
-The SQL argument must select a single column which is interpreted as
-containing the oids of objects to be retrieved. Our code adds
-additional SQL that uses the oids to select objects from _AutoDB.
-
-If you supply search_key=>value pairs and a SQL statement, they are ANDed.
-
 =head3 find
 
  Title   : find
@@ -1228,11 +1056,6 @@ If you supply search_key=>value pairs and a SQL statement, they are ANDed.
            -- OR --
            my $cursor=$autodb->find(collection=>'Person',
                                     query=>{name=>'Joe',sex=>'M'})
-           -- OR --
-           my $cursor=$autodb->find
-           -- OR --
-           my $cursor=$autodb->find(sql=>qq(SELECT oid FROM Person 
-                                            WHERE name='Joe' OR name='Bill'))
  Function: Execute query. 
  Returns : Class::AutoDB::Cursor object which can be used to retrieve results
  Args    : same as 'get'
@@ -1245,11 +1068,6 @@ If you supply search_key=>value pairs and a SQL statement, they are ANDed.
            -- OR --
            my $count=$autodb->count(collection=>'Person',
                                     query=>{name=>'Joe',sex=>'M'})
-           -- OR --
-           my $cursor=$autodb->find
-           -- OR --
-           my $cursor=$autodb->find(sql=>qq(SELECT oid FROM Person 
-                                            WHERE name='Joe' OR name='Bill'))
  Function: Count number of objects satisfying query
  Returns : number
  Args    : same as 'get'
@@ -1262,13 +1080,12 @@ If you supply search_key=>value pairs and a SQL statement, they are ANDed.
  Function: Access object's oid (immutable object identifier)
  Returns : oid as number, NOT Class::AutoDB::Oid object. undef if argument not
            persistent
- Args    : persistent object, Class::AutoDB::Oid object, or 
-           Class::AutoDB::OidDeleted object
+ Args    : persistent object or Class::AutoDB::Oid object
 
 =head2 Cursors
 
 The methods described here operate on Class::AutoDB::Cursor objects
-returned by L<"find">.  See L<METHODS AND FUNCTIONS - Queries|"Queries"> for
+returned by L<"find">.  See L<"METHODS AND FUNCTIONS - Queries"> for
 related methods operating on Class::AutoDB objects.
 
 =head3 get
@@ -1319,8 +1136,7 @@ remaining objects
  Usage   : $autodb->put(@objects)
  Function: Store one or more objects in database
  Returns : nothing
- Args    : list of persistent objects, Oids, or OidDeleteds
- Notes   : nop (does nothing) on Oids and OidDeleteds
+ Args    : list of persistent objects or Oids
 
 The difference between 'put' and 'put_objects' is that when called
 with no objects, 'put' does nothing, while 'put_objects' stores all
@@ -1335,22 +1151,12 @@ persistent objects
  Function: Store all persistent objects (first form) or list of persistent 
            objects (second form)
  Returns : nothing
- Args    : list of persistent objects, Oids, or OidDeleteds
- Notes   : nop (does nothing) on Oids and OidDeleteds
+ Args    : list of persistent objects or Oids
 
 The difference between 'put' and 'put_objects' is that when called
 with no objects, 'put' does nothing, while 'put_objects' stores all
 persistent objects
 
-=head3 del
-
- Title   : del
- Usage   : $autodb->del(@objects)
- Function: Delete one or more objects from memory and database
- Returns : nothing
- Args    : list of persistent objects, Oids, or OidDeleteds
- Notes   : nop (does nothing) on OidDeleteds
-
 =head2 Manage database schema 
 
 =head3 exists
@@ -1434,8 +1240,9 @@ The 'drop' option to 'new' runs this method.
 When you declare a class to be persistent, we arrange for it to
 inherit from Class::AutoDB::Object by modifying its @ISA array. This
 has the effect of polluting your class's namespace with inherited
-methods.  It was a bad idea.  Sorry.  We hope to change this in a
-future release.
+methods.  It was a bad idea.  Sorry.  We plan to change this in a
+future release (currently planned for 1.30; see L<DEVELOPMENT
+ROADMAP>).
 
 Two of the inherited methods are commonly used. These can be invoked
 on a persistent object or a Class::AutoDB::Oid object. It's usually
@@ -1455,7 +1262,6 @@ your objects are thawed before invoking the methods.
  Returns : nothing
  Args    : none
  Notes   : Deprecated.  Should use $autodb->put($object)
-           Legal to invoke on Oid or OidDeleted but does nothing
 
 =head3 oid
 
@@ -1465,7 +1271,6 @@ your objects are thawed before invoking the methods.
  Returns : oid as number, NOT Class::AutoDB::Oid object
  Args    : none
  Notes   : Deprecated.  Should use $autodb->oid($object)
-           Also works on Oid or OidDeleted
 
 =head1 SEE ALSO
 
@@ -1516,24 +1321,35 @@ testing but may be too severe for normal installs.
 You can only have one active AutoDB object at a time.  Grievous errors
 will occur if you run 'new' a second time on a different database.
 
-=item 4. No transactions
+=item 4. No delete operation
+
+This is on our to-do list!  The issue that's held this up is that
+deletion is not a Perl concept -- you never have to delete a Perl
+object -- and it's not obvious how to make it intuitive for
+applications to handle deleted persistent objects.  In our running
+example, suppose you delete a Person Joe who's on Mary's friends list.
+When processing Mary's friends, what should happen to Joe?  Should we
+silently delete him from the list?  Or should we leave him on the list
+but return undef when your program accesses him?  Or something else?
+
+=item 5. No transactions
 
 This is also on our to-do list, but further down, as we have no urgent
 need for transactions in our application.
 
-=item 5. Type mismatches
+=item 6. Type mismatches
 
 If data of the wrong type is stored in a search key, data conversion
 occurs as described in L<Type mismatches>.
 
-=item 6. Class attributes not stored or retrieved
+=item 7. Class attributes not stored or retrieved
 
 Class attributes (as that term is used in L<Class::AutoClass>) are not
 stored in the AutoDB database and are not set when objects are fetched
 or classes 'used'. This is related to the simple way class attributes
 are handled in Class::AutoClass.
 
-=item 7. Uneven error checking
+=item 8. Uneven error checking
 
 The software does not consistently check for user errors. If you make
 a mistake and pass in bad data. the software sometimes responds with
@@ -1541,17 +1357,192 @@ an intelligible error message but often does not.  All too often, the
 mistake winds its way into the guts of the software which ultimately
 dies with an inscrutable message.
  
-=item 8. Pollution of user class namespace
-
-The current design requires that persistent classes be subclasses of
-Class::AutoDB::Object; we ensure this by pushing the required class
-onto @ISA if it's not already there.  This has the unfortunate side
-effect of polluting the persistent classes' namespace with every
-method in Class::AutoDB::Object and above.  This is only a problem
-when user classes employ multiple inheritance.
-
 =back
 
+=head1 DEVELOPMENT ROADMAP
+
+Subject to change, of course.  You will notice, I'm sure, that no dates are attached to releases.
+
+=begin html
+
+<h2>1.20 database operations</h2>
+<h3>'del' operation - a bit messy since object deletion is not a Perl concept!</h3>
+<ul>
+<li>messy since object deletion is not a Perl concept!</li>
+<li>similar issues aries with weak references. I think Perl causes deref of lost reference to return undef</li>
+<ul>
+<li>this has to complicate the application (see notes below).  but is there a better choice??</li>
+<li>See Hash::NoRef for weak cache -- not sure if this comment is still relevant</li>
+</ul>
+<li>Database part is easy: delete object from object table and all collections (should consult saved registry for full list of collections that may contain the object)</li>
+<li>Object cache is a little tricky: if the oid is present in the cache as an Object, deflate it to an Oid; if it's present as an Oid, do nothing.</li>
+<li>Hard part is that other in-memory or in-database objects may contain references to the deleted object.</li>
+<ul>
+<li>Serialize::fetch handles this case fine - it returns undef</li>
+<li>Have to check everywhere in our code that does 'fetch' or 'really_fetch' to make sure undef is handled.</li>
+<li>One place is in Cursor::get - obvious action is to skip undefs</li>
+<li>Harder case is Oid::AUTOLOAD - the simplest action would be to return undef to the application, but this is misleading: the app said something like $obj-&gt;xxx expecting to get the 'xxx' attribute; giving him undef suggests that 'xxx' is undef, not that $obj is deleted!  Perhaps we should throw an exception, but can the app catch it??</li>
+</ul>
+</ul>
+<h3>Easy database things (raw SQL in 'find' and other improvements to 'find' queries)</h3>
+<ul>
+<li>Allow raw SQL</li>
+<ul>
+<li>Extremely easy - the code is already set up to do it.  See Cursor::select_sql and Cursor::count_sql</li>
+</ul>
+<li>Allow arbitrary MySQL types</li>
+<ul>
+<li>Should be done in conjunction with cleaning up the type stuff in Table</li>
+</ul>
+<li>AND'ing new query onto Cursor</li>
+<li>let empty query retrieve all objects.  ie $autodb-&gt;get</li>
+<li>DBI::quote seems to be quoting numbers - I wonder if this is correct</li>
+</ul>
+<h3>consider new serialization schemes</h3>
+<ul>
+<li>want to handle ties. at least, Hash::AutoHash and friends</li>
+<li>Storable may be viable</li>
+<ul>
+<li>looks like new 'attach' hook gives us what we need. another plus is that it handles ties.</li>
+<li>a plus is that Storable handled ties</li>
+<li>problem is that output is not human readable</li>
+</ul>
+<li>do a combination of Storable for the heavy lift and DD to produce readable form</li>
+</ul>
+<h3>Table</h3>
+<ul>
+<li>Refactor TYPES - currently duplicated in several modules - a real landmine waiting to go off!</li>
+<li>Provide utility methods for working with types</li>
+<li>Might be better to let Table do the database ops itself rather than just generating SQL - doesn't matter much now, but may make statement handles easier to manage</li>
+</ul>
+<h3>add 'configure' to Build process</h3>
+<h2>1.30 namespace issues</h2>
+<h3>clean up namespace imposed on user-objects (maybe in AutoClass, too)</h3>
+<ul>
+<li>we currently splice Class::AutoDB::Object into user-class's @ISA.  Object, in turn, inherits from Serialize.  this is a bad idea that puts a lot of methods in user namespace</li>
+<ul>
+<li>the code depends on this, among other things, to know whether a class is persistent. the code also uses this to gain access to methods like 'fetch' and 'thaw'</li>
+<li>we want to stop this, which will, of course, require finding another way to tell whether a class is persistent and to invoke 'fetch', 'thaw', etc.</li>
+</ul>
+<li>it would be nice to continue providing convenience methods in user namespace: 'put', 'oid', and maybe 'autodb'.</li>
+<ul>
+<li>it must be okay for user to override these, which means our code must never call them. I think this is already the case</li>
+<li>the hard problem is what to do when invoked on Oids. </li>
+<li>presently, the Oid versions of these are hardcoded. to do it right, Oid code has to check whether user has overridden the method; if so, has to fetch and re-dispatch to user object</li>
+<li>instead of, or in addition to, 'autodb' method on user-objects, have AutoDB provide this as an exported function.</li>
+<ul>
+<li>think though implications when we allow multiple active AutoDB objects</li>
+</ul>
+</ul>
+</ul>
+<h3>AutoDB methods to list objects (or maybe oids??) -- this is a 'maybe' item</h3>
+<ul>
+<li>mem_objects -- all persistent objects in memory -- these are the ones that will be put by put_objects</li>
+<li>db_objects -- objects in database -- gets 'em all as Oids</li>
+<li>all_objects -- union of mem+db</li>
+<li>mem_only_objects (or maybe new_objects) -- objects in memory but not yet in db</li>
+<li>db_only_objects</li>
+<li>del_objects -- someday -- deleted objects -- here is where 'oids' makes more sense than 'objects'</li>
+</ul>
+<h2>1.40 improved put/get; performance improvements</h2>
+<h3>New thoughts on 'put' -- batch 'put' is super performance win</h3>
+<ul>
+<li>Improve efficiency of put_objects method and add ability to feed in list of objects. Should really be done as part of put-policy</li>
+<ul>
+<li>By default, $ob-&gt;put doesn't write the object immediately, but rather puts it on a queue for later writing</li>
+<li>$autodb-&gt;put does actual writes. Called automatically by $autodb-&gt;close and maybe destructor if we can get this to work.</li>
+<li>In the full scheme of things, 'put' can specify queue so that objects can be written at different times. This could grow into a limited transaction capability.</li>
+<li>$obj-&gt;put could also specify whether current state of object should be written, or state as it exists when $autodb-&gt;put is called. The latter option is really another way of saying, “$obj is dirty” and is a backdoor way of approximating auto-put.</li>
+<li>For performance, $autodb-&gt;put presumably will sort database operations by table and do updates in batches</li>
+<li>Ideally want to group updates by table and do each table in bulk.</li>
+<li>This is going to be a pain - have to refactor code to collect SQL bits and pieces and put them together at the top. Will affect a lot of tests…</li>
+<li>Here's a plausible design.</li>
+<ul>
+<li>Each table has methods to</li>
+<li>also need to collect list of tables with pending updates. This could be attached to the autodb</li>
+</ul>
+<li>Updates to _AutoDB.</li>
+<ul>
+<li>Class::AutoDB::Object:::put calls Class::AutoDB::Serialize::store on each object</li>
+</ul>
+<li>Updates to Collections</li>
+<ul>
+<li>Class::AutoDB::Object:::put calls Collection::put on each object which calls BaseTable::put and ListTable::put</li>
+</ul>
+</ul>
+<li>'dirty bit' put policy</li>
+<ul>
+<li>May be better to think of this as 'deferred put'.</li>
+<li>When code decides that object is dirty, puts it on a list of objects to be 'put'</li>
+<li>Later, perhaps during $autodb-&gt;put, the actual database updates occur</li>
+</ul>
+<li>Look at renaming put_objects to 'put'. This will simply work.  Just need synonym and add to tests</li>
+<li>re-get (or maybe refresh) to get current value of object from database</li>
+</ul>
+<h3>Performance improvements for 'get'</h3>
+<ul>
+<li>may be useful to let object provide list of "related objects" and get them all at once</li>
+</ul>
+<h3>Use statement handles in Serialize and Table to improve performance</h3>
+<h3>design concurrency/transaction scheme -- will be implemented later</h3>
+<h2>1.50 better collections; class extents, etc.</h2>
+<h3>class attributes </h3>
+<ul>
+<li>not set when class 'used' in the course of fetching objects. see discussion under AutoClass</li>
+<li>AutoDB does not store class attributes</li>
+<li>related to simple way class attributes handled in AutoClass</li>
+</ul>
+<h3>Test subclass w/o collection, i.e., transients only</h3>
+<h3>Class extents</h3>
+<ul>
+<li>implement as more or less standard list table, e.g. _AutoDB_classes(class,oid)</li>
+<li>Specify via a flag in &#37AUTODB</li>
+<li>subclasses have to insert in parent extents</li>
+</ul>
+<h2>1.60 multiple AutoDBs</h2>
+<h3>Allow program to have multiple AutoDB objects, connected to different databases</h3>
+<h3>Objects fetched from a given database are, by default, put back there</h3>
+<h3>Probably also possible for 'put' to specify which database gets the object</h3>
+<h3>Big problem is schema registration. Presently, schemas are auto- registered in default database.  With new scheme, how does program conveniently control this?</h3>
+<h2>1.70 oid management</h2>
+<h3>Current scheme -- generate unique id in Perl, use MySQL 'replace' to insert or update.  (Note: MySQL's 'insert… on duplicate key update' is a better choice, but it's only available from 4.1)  Problems:</h3>
+<ul>
+<li>The database will be seriously corrupted if a new oid isn't unique!!</li>
+<li>'replace' isn't available in Postgres; will either need extra 'select' or stored procedure -- is this still true?</li>
+<li>'replace' is available in SQLite</li>
+</ul>
+<h3>Alternative - get sequence number from database</h3>
+<h3>Another goal: ensure uniqueness of oids constructed on different machines, so databases can be safely merged...</h3>
+<h3>Option 1 - do it when object constructed</h3>
+<ul>
+<li>Problem - extra trip to database</li>
+<li>Benefit - every update to object (even first) is a database 'update' - works for MySQL and Postgres</li>
+</ul>
+<h3>Option 2 - do it when object stored</h3>
+<ul>
+<li>Benefit - avoids extra trip to database</li>
+<li>Problem - self-referential objects are complicated.  If freeze code encounters the same object twice during serialization, there won't be an oid yet.  The obvious recursion (if object doesn't have oid, then store it) leads to infinite loop.  Probably need to maintain a hash of 'in-progress' objects; if freeze encounters an in-progress object, it has to do a dummy insert just to get the oid.</li>
+</ul>
+<h2>1.80 persistent big HASH and ARRAY</h2>
+<h3>persistent HASH</h3>
+<h3>persistent ARRAY --  harder because of shifts &amp; splices!!</h3>
+<ul>
+<li>(see also implementation of BerkeleyDB recno files)</li>
+<li>The obvious implementation is to give ARRAY elements an 'array_index' key, starting at 0.</li>
+<li>So long as you just do pushes and pops, the array will grow and shrink on high end, and the mapping from Perl array index to database array index is trivial</li>
+<li>But, when a shift is done, the element at the low end disappears and now the 0th Perl element has database index 1</li>
+<li>So, we need a translation mechanism that maps Perl index values into database index values</li>
+<li>In general, splice can introduce 'holes' and 'bumps' into the mapping: e.g., if you delete 10 elements from the middle of the ARRAY, the bordering Perl index values will differ by 10 in the database; or if you add 10 elements to the middle of the ARRAY, the database indexes assigned to these guys will be discontinuous from their neighbors</li>
+<li>Blech…</li>
+<li>Possible implementation: translation table is a list of &lt;count, database index&gt; pairs.  Meaning is that the first/next count elements have database indexes index, index+1,…, index+count-1</li>
+<li>In usual case (no general splices), table has one entry</li>
+</ul>
+<h3>make sure design will work with concurrency</h3>
+<h2>2.00 port to SQLite or other DBMS</h2>
+<h2>2.10 concurrency/transactions</h2>
+
+=end html
+
 =head1 SUPPORT
 
 You can find documentation for this module with the perldoc command.
